@@ -1,29 +1,31 @@
 // CLI used for testing the dovecot authentication package.
 //
 // NOT for production use.
+// +build !coverage
+
 package main
 
 import (
+	"flag"
 	"fmt"
-	"os"
 
 	"blitiri.com.ar/go/chasquid/internal/dovecot"
 )
 
 func main() {
-	a := dovecot.NewAuth(os.Args[1]+"-userdb", os.Args[1]+"-client")
+	flag.Parse()
+	a := dovecot.NewAuth(flag.Arg(0)+"-userdb", flag.Arg(0)+"-client")
 
 	var ok bool
 	var err error
 
-	switch os.Args[2] {
+	switch flag.Arg(1) {
 	case "exists":
-		ok, err = a.Exists(os.Args[3])
+		ok, err = a.Exists(flag.Arg(2))
 	case "auth":
-		ok, err = a.Authenticate(os.Args[3], os.Args[4])
+		ok, err = a.Authenticate(flag.Arg(2), flag.Arg(3))
 	default:
 		fmt.Printf("unknown subcommand\n")
-		os.Exit(1)
 	}
 
 	if ok {
@@ -32,5 +34,4 @@ func main() {
 	}
 
 	fmt.Printf("no: %v\n", err)
-	os.Exit(1)
 }
